/**
 * JavaScript hooks for the sp-link macro in the Confluence 4 Editor
 **/
AJS.Confluence.PropertyPanel.Macro.registerButtonHandler("view-link-in-sp", function(e, macroNode) {

    var rawParams = AJS.$(macroNode).attr("data-macro-parameters").split("|");
    AJS.$.each(rawParams, function(index, param)
    {
        var splitIndex = param.indexOf("=");
        if (splitIndex > 0)
        {
            var key = param.substring(0, splitIndex);
            var value = param.substring(splitIndex + 1);

            if (key == "url")
            {
                window.open(value);
                return false;
            }
        }
    });
});

(function($) {
    var spLinkEditor = function(data) {
        AJS.Rte.BookmarkManager.storeBookmark();

        // Launch our custom UI to do stuff.
        var popup = new AJS.Dialog(860, 530);
        popup.addHeader(AJS.I18n.getText("com.atlassian.confluence.extra.sharepoint.sp-link.dialog.title"));
        popup.addPanel(AJS.I18n.getText("com.atlassian.confluence.extra.sharepoint.sp-link.dialog.panel.title"), Confluence.Templates.SharePoint.searchDialog({}), "panel-body");

        var searchField = $("#sharepoint-document-search-query");
        if (data.params)
        {
            searchField.val(data.params.title);
            // do a search.
        }
        searchField.focus(function() {
            var me = $(this);
            if (me.hasClass("placeholded"))
            {
                me.removeClass("placeholded");
                me.val("");
            }
        });
        searchField.blur(function() {
            var me = $(this);
            if (me.val() == "")
            {
                me.addClass("placeholded");
                me.val("Search");
            }
        });

        popup.addSubmit(AJS.I18n.getText("com.atlassian.confluence.extra.sharepoint.sp-link.dialog.button.insert"), function(dialog) {
            $("#execute-sharepoint-search").unbind("click");

            var selectedItem = $("tr.selected td.name");
            var linkText = selectedItem.text();
            var url = selectedItem.attr("data-url");

            popup.hide();
            popup = null;
            dialog.remove();

            // Done
            var macroRenderRequest = {
                contentId: AJS.Meta.get('content-id') || "0",
                macro: {
                    name: "sp-link",
                    params: {
                        url: url,
                        title: linkText,
                        defaultParameterValue: "",
                        body: ""
                    }
                }
            };

            AJS.Rte.BookmarkManager.restoreBookmark();

            var t = tinymce.confluence.macrobrowser;
            if (t.editedMacroDiv) {
                tinymce.confluence.MacroUtils.insertMacro(macroRenderRequest, t.editedMacroDiv);
                delete t.editedMacroDiv;
            } else {
                tinymce.confluence.MacroUtils.insertMacro(macroRenderRequest);
            }

        }, "sp-link-button-ok");
        popup.addCancel(AJS.I18n.getText("com.atlassian.confluence.extra.sharepoint.sp-link.dialog.button.cancel"), function(dialog) {
            $("#sharepoint-link-search").unbind("click");
            popup.hide();
            popup = null;
            dialog.remove();
        });

        popup.show();
        $(".button-panel-submit-button").attr("disabled", "true");

        var bindExistingAlias = undefined;
        if (data.params) {
            var existingUrl = data.params.url;
            bindExistingAlias = function(originalValue, sites, defaultSite) {

                var match;
                jQuery.each(sites, function(index, site) {
                    if (startsWith(site.serverUrl, existingUrl))
                    {
                        match = site;
                        return false;
                    }
                });

                return match;

                function startsWith(snippet, full)
                {
                    return full.substring(0, snippet.length) === snippet;
                }
            };
        }
        AJS.SharePoint.bindSitesDropDown($("#sharepoint-site-alias"), undefined, bindExistingAlias);

        var showSpinner = function (element, radius, centerWidth, centerHeight) {
            AJS.$.data(element, "spinner", Raphael.spinner(element, radius, "#666"));
            // helps with centering the spinner
        };
        var hideSpinner = function (element) {
            AJS.$(element).css('marginTop','');
            AJS.$(element).css('marginLeft', '');
            var spinner = AJS.$.data(element, "spinner");
            if (spinner) {
                spinner();
                delete spinner;
                AJS.$.data(element, "spinner", null);
            }
        };

        searchField.parents("form").submit(function(event) {
            $(".button-panel-submit-button").attr("disabled", "true");

            var results = $(".sharepoint-search-results");

            var queryField = $("#sharepoint-document-search-query");
            if (queryField.hasClass("placeholded") || queryField.val() == "")
            {
                results.html(Confluence.Templates.SharePoint.searchError({message: AJS.I18n.getText("com.atlassian.confluence.extra.sharepoint.sp-link.dialog.error.noQuery")}));
                event.preventDefault();
                return false;
            }

            results.html("<div class=\"spinner\" style=\"width: 122px; height: 122px; margin-left: auto; padding-top: 70px; margin-right: auto;\"></div>");
            var spinner = $(".sharepoint-search-results .spinner");
            showSpinner(spinner[0], 50, true, true);


            $.ajax({
                url: AJS.format("{0}/rest/sharepoint/1.0/search/doc?siteAlias={1}&query={2}",
                                AJS.Confluence.getContextPath(),
                                encodeURIComponent($("#sharepoint-site-alias").val()),
                                encodeURIComponent(queryField.val())),
                dataType: "json",
                timeout: 30000, // 30 seconds.
                success: function(data) {
                    AJS.log(data);
                    $(".sharepoint-search-results").html(Confluence.Templates.SharePoint.searchResults({searchResults: data.contentNameMatches[0]}));
                },
                error: function(xhr, text, error) {
                    AJS.log("Failed to search SharePoint:(" + text + ") " + error);
                    $(".sharepoint-search-results").html(Confluence.Templates.SharePoint.searchError({ message: null} ));
                }
            });
            event.preventDefault();
            return false;
        });


        $("table.sharepoint-data tr").live("click", function() {
            if ($(this).hasClass("data-table-header"))
                return;

            $("table.sharepoint-data tr.selected").removeClass("selected");
            $(this).addClass("selected");

            $(".button-panel-submit-button").removeAttr("disabled");
        });
    };

    AJS.MacroBrowser.setMacroJsOverride("sp-link", {opener: spLinkEditor});
})(AJS.$);
